/* DiaCanvas -- A technical drawing canvas.
 * Copyright (C) 1999, Arjan Molenaar
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* diamodifytool.h
 * ---------------
 * The DiaModifyTool is responsible for forwarding events to the DiaObject
 * event handler and handling the requests the object places in its REQUEST
 * field.
 */

#ifndef __DIA_MODIFY_TOOL_H__
#define __DIA_MODIFY_TOOL_H__

#include <diacanvas/diabasemodifytool.h>

DIA_OPEN

typedef struct _DiaModifyTool DiaModifyTool;

//enum DiaModifyState {
  //DIA_STATE_NONE,
    //DIA_STATE_BOX_SELECT
    //};

struct _DiaModifyTool
{
  DiaBaseModifyTool tool;

  //enum DiaModifyState state;

  //Point start_box;
  //Point end_box;
  //  DiaObject *box;
  
};

void dia_modify_tool_init (DiaModifyTool *tool);

DiaTool* dia_modify_tool_new ();


DIA_CLOSE

#endif /* MODIFY_TOOL_H */






