/* DiaCanvas -- A technical drawing canvas.
 * Copyright (C) 1999, Arjan Molenaar
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* diamagnifytool.h
 * ----------------
 * Easy tool for zooming and unzooming the canvas. This can be done by
 */

#ifndef __DIA_MAGNIFY_TOOL_H__
#define __DIA_MAGNIFY_TOOL_H__

#include <diacanvas/geometry.h>
#include <diacanvas/diatool.h>
#include <diacanvas/diadefs.h>

DIA_OPEN

typedef struct _DiaMagnifyTool DiaMagnifyTool;


struct _DiaMagnifyTool {
  DiaTool tool;

  //gboolean box;
  
  //Point start_box;
  //Point end_box;
};

DiaTool* dia_magnify_tool_new ();

DIA_CLOSE

#endif /* MAGNIFY_TOOL_H */


