/* DiaCanvas -- A technical drawing canvas.
 * Copyright (C) 1999, Arjan Molenaar <arjan@xirion.nl>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This file is generated by make-header.sh, so not edit by hand.
 */

#ifndef __DIA_CANVAS_H__
#define __DIA_CANVAS_H__

#include <diabaseelement.h>
#include <diabaseline.h>
#include <diabasemodifytool.h>
#include <diabasetext.h>
#include <diabox.h>
#include <diaboxelement.h>
#include <diacolor.h>
#include <diacreatetool.h>
#include <diadefs.h>
#include <diadiagram.h>
#include <diadisplay.h>
#include <diadynelement.h>
#include <diadynline.h>
#include <diaelement.h>
#include <diaentry.h>
#include <diaevent.h>
#include <diafont.h>
#include <diagraph.h>
#include <diagrid.h>
#include <diagroup.h>
#include <diaimage.h>
#include <dialayer.h>
#include <diamagnifytool.h>
#include <diamain.h>
#include <diamodifytool.h>
#include <diamultiline.h>
#include <diaobject.h>
#include <diarenderer.h>
#include <diarenderereps.h>
#include <diarenderergdk.h>
#include <diatool.h>
#include <geometry.h>

#endif /* __DIA_CANVAS_H__ */
