/* DiaCanvas -- a technical canvas widget
 * Copyright (C) 1999 Arjan Molenaar
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __DIA_MAIN_H__
#define __DIA_MAIN_H__

#include "diadefs.h"
#include <glib.h>

DIA_OPEN

#define DIA_CANVAS_MAJOR_VERSION (0)
#define DIA_CANVAS_MINOR_VERSION (40)
#define DIA_CANVAS_MICRO_VERSION (1)

extern const gchar* dia_canvas_current_version;

void dia_init ();

DIA_CLOSE

#endif





